#!/usr/bin/perl

#Murasaki - multiple genome global alignment program
#Copyright (C) 2006-2007 Keio University
#(Kris Popendorf) <comp@bio.keio.ac.jp> (2006)
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

use Getopt::Long;
use Pod::Usage;
use File::Basename;
use IO::Handle;
use Data::Dump qw {dump};

use Ticker qw{resetTick tick};
use KOG;

use strict;

our $root='/home/krisp/murasaki';
our $seqhome="$root/seq";
our $koglist;

our ($help,$man);
our ($kogfile,@kogmap);
GetOptions('help|?' => \$help, man => \$man,
	   'kogfile=s' => \$kogfile,
	   'kog=s%' => sub {pod2usage(1)
			      unless($_[1] and $_[2] and $_[1]>0 and $_[2]=~m/^...$/);
			    $kogmap[$_[1]]=$_[2]},
	  'koglist=s' => \$koglist);
pod2usage(1) if $help or $#ARGV<0;
pod2usage(-exitstatus => 0, -verbose => 2) if $man;

die "Need to specify kogfile with --kogfile" unless $kogfile;

if($koglist){
  my $count;
  open(my $koglistfh,$koglist) or die "Couldn't open $koglist";
  $koglist={};
  while(<$koglistfh>){
    chomp;
    $koglist->{$_}=1;
    $count++;
  }
  print "\"Used members\" list loaded. $count approved locs.\n";
}

my $outfile=shift @ARGV;
if($outfile=~m/\.seqs$/){ #alignment file
  my $basename=$`;
  print "Loading sequence list from $outfile\n";
  open(INF,$outfile);
  while(<INF>){
    chomp;
    push(@ARGV,$_);
  }
  close(INF);
  $outfile="$basename.kogaligned";
  print "Storing output to $outfile\n";
}

our (@locusLists,%knownLocs);

my ($i);
our @knownKogs=(KOG->knownKogs,KOG->knownCogs);
my $kogre=join("|",@knownKogs);
my %aliases=KOG->commonAliases;

LoadSeqs: foreach my $seq (@ARGV){
  unless($kogmap[$i]){
    local $_=$seq;
    my ($id)=m/($kogre)/;
    unless($id){ #try harder...
      foreach my $alias (keys(%aliases)){
	$id=$aliases{$alias} if (m/$alias/i);
      }
    }
    if($id){
      print "Identified $_ as KOG member $id\n";
      $kogmap[$i]=$id;
    }
  }
  die "$seq isn't bound to a kog. Use --kog (eg. --kog $i=MtC) to set it.\n"
    unless $kogmap[$i];
  if(!-f "$seq.cds"){
    print "CDS file not found for $seq. Generating...\n";
    my $res=system("$root/getcds.pl $seq");
    do {
      die "Generation of CDS file for $seq failed" unless -f "$seq.cds\n";
      next LoadSeqs;
    } unless -f "$seq.cds";
  }
  open(CDS,"$seq.cds") or die "Couldn't read $seq.cds";
  print "Loading annotation for $seq...";
  my ($cdscount);
 LoadCDS: while(<CDS>){
    my ($name,$start,$stop,$strand,$locus) = split(/\s/,$_);
    $locus=$name if $name && !$locus;
    next unless $locus;
    $locus=uc $locus;
    my $cd={name => $name,start=>$start,stop=>$stop,strand=>$strand,locus=>$locus};
    print "Uh oh, $locus occurs again!\n" if $locusLists[$i]->{$locus};
    $locusLists[$i]->{$locus}=$cd;
    $knownLocs{$locus}=1;
    $cdscount++;
  }
  print("$cdscount CDS's loaded.\n");
}continue{$i++}

my $kogs=KOG->kogFrom($kogfile,\@kogmap,\%knownLocs);

open(OUTF,">$outfile") or die "Couldn't open $outfile for writing\n";
our $count=0;
resetTick(scalar(keys(%{$locusLists[0]})));
foreach my $locus (keys(%{$locusLists[0]})){
  my $allmembers=$kogs->memberMap($locus);
  my @bitsToPlot;
  my $i=0;
  foreach my $spec (@kogmap){
    $bitsToPlot[$i]=[grep {$allmembers->{$_} eq $spec} keys(%$allmembers)];
  }continue{$i++}
  plotBits(0,"",@bitsToPlot);
  tick();
}
close OUTF;
print "\nDone. $count psuedo-anchors.\n";

sub plotBits {
  my ($ite,$leaderBits,$mybits,@others)=@_;
  unless($mybits){
    print OUTF $leaderBits."\n";
    $count++;
    return;
  }
  foreach my $bit (@$mybits){
    local $_=$locusLists[$ite]->{$bit};
    if($koglist){
      next unless $koglist->{$_->{locus}};
    }
    $leaderBits.="\t" if $leaderBits;
    plotBits($ite+1,($leaderBits.join("\t",@{$_}{qw{start stop}},$_->{strand}<0 ? "-":"+")),@others);
  }
}

__END__

=head1 NAME

cdsalign.pl -- produce (untested) alignments from cds files

=head1 SYNOPSIS

cdalign.pl <output> <input1> <input2> [<input3> ...]
cdalign.pl <alignment.seq>

=head1 OPTIONS
The cdalign.pl <aligment.seq> execution style writes results to
align.kogaligned

=cut
